/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2005,2006          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/*
 * "@(#) 1.6 src/rsct/lapi/pnsd/ntbl2.h, lapi, comm_rco, rco0626a 06/09/05 17:13:29"
 *
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * 5765-296
 * 5765-529
 * 5765-G2601
 *
 * (C) Copyright IBM Corp. 2005
 *
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 */

/*
 * Start Date     : October, 2005
 * Change History :
 *
 *   2005
 *   ----
 *   11/22: ALL F129208 - PNSD initial drop
 *   11/28:  YZ F118247 - change the name of windown state
 *   11/29:  HX D131486 - fix misalignment in adap_resources_t
 *   12/06:  RS D131645 - Make per_task_input for load table a single pointer
 *
 *   2006
 *   ----
 *    1/27:  RS D132734 - Added new error code NTBL2_NTBL_ALREADY_LOADED
 */
#ifndef _NTBL2_H
#define _NTBL2_H

#include <stdint.h>

#define MAX_SPIGOTS               4
#define NTBL2_MAX_DEVICENAME_SIZE 32
#define NTBL2_VERSION             420
typedef enum { LEAVE_IN_USE, KILL } clean_option_t;

////////////////////////////////////////////////////////////////////
// Following structures represent values that must be obtained from 
// the resource manager. The *_per_table_input_t structure actually 
// represents values passed in as parameters to the NTBL API calls.
//////////////////////////////////////////////////////////////////// 
typedef struct {              
    /////////////////////////////////////////////////////////////////
    // Structure provided by LL as part of Load NTBL for each task 
    //     for IB.
    /////////////////////////////////////////////////////////////////
    uint16_t task_id;
    uint16_t win_id;
    uint32_t node_number;                            // provided by HA
    char     device_name[NTBL2_MAX_DEVICENAME_SIZE]; // e.g, /dev/hpce0

    uint16_t base_lid;
    uint8_t  port_id;
    uint8_t  lmc;
    uint8_t  port_status;  // ??? need this for IB failover/recovery ???
    uint8_t  padding[3];
} ntbl_creator_ib_per_task_input_t;

typedef struct {
    /////////////////////////////////////////////////////////////////////////
    // Structure encapsulating info provided by LL as part of Load NTBL for
    //     HPCE for each task.
    /////////////////////////////////////////////////////////////////////////
    uint16_t task_id;
    uint16_t win_id;
    uint32_t node_number;                            // provided by HA
    char     device_name[NTBL2_MAX_DEVICENAME_SIZE]; // e.g, /dev/hpce0
} ntbl_creator_hpce_per_task_input_t;
   
typedef union {
    ntbl_creator_hpce_per_task_input_t hpce_per_task;
    ntbl_creator_ib_per_task_input_t   ib_per_task;
} ntbl_creator_per_task_input_t;

typedef struct {
    /////////////////////////////////////////////////////////////////////////
    // Structure (or info) provided by LL as part of Load NTBL (one per NTBL)
    //     for IB and HPCE. Note the prefix ntbl_creator* is chosen so as to 
    //     be LL independent.
    /////////////////////////////////////////////////////////////////////////
    uint16_t num_tasks;
    uint16_t job_key;
    uid_t    uid;
    uint64_t net_id;
} ntbl_creator_per_table_input_t;

//
// Following structure is used in ntbl2_adapter_resources() call
//
typedef struct {
    uint32_t node_number;             // to form unique ID
    uint8_t  num_spigots;             // Ports for IB, spigots for HPCE
    uint8_t  padding1[3];             // future use?
    uint16_t lid[MAX_SPIGOTS];        // (IB only) Logical ID of each spigot    
    uint64_t network_id[MAX_SPIGOTS]; // Network ID for each adapter spigot 
    uint8_t  lmc[MAX_SPIGOTS];        // (IB only) Logical mask of each spigot 
    uint8_t  spigot_id[MAX_SPIGOTS];  // Port/Spigot IDs
    uint16_t window_count;            // Count of windows in window_list 
    uint8_t  padding2[6];             // future use?
    uint16_t *window_list;            // array of available windows         
    uint64_t rcontext_block_count;    // available rcxt blocks    
} adap_resources_t;

//
// Structure to request and get info about any (arbitrarily
// selected) reserved window 
//
typedef struct {
    uint32_t node_number;  // to fill in uniq_id structure
    uint8_t  padding[4];  
    char     device_name[NTBL2_MAX_DEVICENAME_SIZE]; 
                           // to fill in uniq_id structure
       // Note: above needs to be full path (path prefix + device name in NAM)
       //     because of ConfigRM which gives full path to LL 9
    uint64_t net_id;
} any_window_info_t;

//
// Following two structures are for the ntbl2_status_adapter() call
//
typedef enum { NTBL2_WIN_UNAVAILABLE, // Initialization stuff in progress
               NTBL2_WIN_INVALID,     // Not a usable window 
               NTBL2_WIN_AVAILABLE,   // Ready for NTBL load
               NTBL2_WIN_RESERVED,    // Window reserved, NTBL not loaded 
                                //     (for POE-PMD)
               NTBL2_WIN_READY,       // NTBL loaded
               NTBL2_WIN_RUNNING      // HAL has gotten NTBL from us
} win_state_t;

typedef struct {
        pid_t       client_pid;      // Pid of process that loaded
        uid_t       uid;             // Uid using the window
        uint16_t    window_id;       // Window being reported
        uint16_t    bulk_transfer;   // Is this load using RDMA?
        uint32_t    rcontext_blocks; // rcontexts per window
        win_state_t state;           // Window state
        uint8_t     padding[4];
} ntbl2_status_t;

typedef enum {
    NTBL2_SUCCESS,            // 0 - Success
    NTBL2_EINVAL,             // 1 - Invalid argument
    NTBL2_EPERM,              // 2 - Caller not authorized
    NTBL2_PNSDAPI,            // 3 - PNSD API returned an error
    NTBL2_EADAPTER,           // 4 - Invalid adapter
    NTBL2_ESYSTEM,            // 5 - System Error occurred
    NTBL2_EMEM,               // 6 - Memory error
    NTBL2_EIO,                // 7 - Adapter reports down
    NTBL2_NO_RDMA_AVAIL,      // 8 - No RDMA windows available 
    NTBL2_EADAPTYPE,          // 9 - Invalid adapter type
    NTBL2_BAD_VERSION,        // 10- Version must match ntbl2_version()
    NTBL2_EAGAIN,             // 11- Try the call again later
    NTBL2_WRONG_WINDOW_STATE, // 12- Window in wrong state for this operation
    NTBL2_UNKNOWN_ADAPTER,    // 13- One (or more) adapters in NTBL unknown
    NTBL2_NO_FREE_WINDOW,     // 14- For reserve any windows (none free)
    NTBL2_NTBL_ALREADY_LOADED // 15- NTBL with same key and task 0 config
                              //     already loaded for this device 
} ntbl2_rc_t;

#ifdef _cplusplus
extern "C" {
#endif // _cplusplus
int ntbl2_version(void);
int ntbl2_unload_window(int version, char *adapter_device_string, 
                        uint16_t adapter_type, ushort job_key, 
                        ushort window_id);
int ntbl2_clean_window(int version, char *adapter_device_string,
                       uint16_t adapter_type,
                       clean_option_t leave_inuse_or_kill, 
                       ushort window_id);
int ntbl2_rdma_jobs(int version, char *adapter_device_string, 
                    uint16_t adapter_type, uint16_t *job_count,
                    uint16_t **job_keys);
int ntbl2_load_table_rdma(int version, char *adapter_device_string,
                          uint16_t adapter_type,
                          uint64_t network_id, uid_t uid, pid_t pid, 
                          ushort job_key, char *job_description, 
                          uint use_bulk_transfer, 
                          uint bulk_transfer_resources, int table_size,   
                          ntbl_creator_per_task_input_t *per_task_input);
int ntbl2_adapter_resources(int version, char *adapter_device_string,
                            uint16_t adapter_type,
                            adap_resources_t *adapter_info_OUT);
int ntbl2_reserve_any_window(int               version, 
                             uint16_t          adapter_type, 
                             uid_t             uid,
                             uint16_t          job_key,
                             uint16_t          num_windows_requested,
                             uint8_t           instances, // unused
                             any_window_info_t *reserved_win_info_OUT,
                             uint16_t          *win_list_OUT);
int ntbl2_status_adapter(int version, char *adapter_device_string, 
                         uint16_t adapter_type, uint16_t *window_count,
                         ntbl2_status_t **status_array);
#ifdef _cplusplus
}
#endif // _cplusplus

typedef int (*ntbl2_version_t)(void);
typedef int (*ntbl2_unload_window_t)(int version, char *adapter_device_string, 
                        uint16_t adapter_type, ushort job_key, 
                        ushort window_id);
typedef int (*ntbl2_clean_window_t)(int version, char *adapter_device_string,
                       uint16_t adapter_type,
                       clean_option_t leave_inuse_or_kill, 
                       ushort window_id);
typedef int (*ntbl2_rdma_jobs_t)(int version, char *adapter_device_string, 
                    uint16_t adapter_type, uint16_t *job_count,
                    uint16_t **job_keys);
typedef int (*ntbl2_load_table_rdma_t)(int version, char *adapter_device_string,
                          uint16_t adapter_type,
                          uint64_t network_id, uid_t uid, pid_t pid, 
                          ushort job_key, char *job_description, 
                          uint use_bulk_transfer, 
                          uint bulk_transfer_resources, int table_size,   
                          ntbl_creator_per_task_input_t *per_task_input);
typedef int (*ntbl2_adapter_resources_t)(int version, 
                            char *adapter_device_string,
                            uint16_t adapter_type,
                            adap_resources_t *adapter_info_OUT);
typedef int (*ntbl2_reserve_any_window_t)(int version, 
                             uint16_t          adapter_type, 
                             uid_t             uid, 
                             uint16_t          job_key, 
                             uint16_t          num_windows_requested,
                             uint8_t           instances, // unused
                             any_window_info_t *reserved_win_info_OUT, 
                             uint16_t          *win_list_OUT);
typedef int (*ntbl2_status_adapter_t)(int version, char *adapter_device_string, 
                         uint16_t adapter_type, uint16_t *window_count,
                         ntbl2_status_t **status_array);
#endif // not _NTBL2_H
